﻿using Microsoft.AspNetCore.Authentication.JwtBearer;
using Microsoft.AspNetCore.Authorization;
using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using System;
using System.Linq;
using System.Security.Claims;
using System.Threading.Tasks;

using TestLibrary.Models;
using TestServer.Database;
using TestServer.Services;

namespace TestServer.Controllers
{
    /// <summary>
    /// 인증 컨트롤러
    /// </summary>
    [ApiController]
    [Route("api/[controller]")]
    public class AuthenticationController : ControllerBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 데이터베이스 컨텍스트
        /// </summary>
        private readonly DatabaseContext databaseContext;
        
        /// <summary>
        /// 토큰 빌더
        /// </summary>
        private readonly ITokenBuilder tokenBuilder;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - AuthenticationController(databaseContext, tokenBuilder)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="databaseContext">데이터베이스 컨텍스트</param>
        /// <param name="tokenBuilder">토큰 빌더</param>
        public AuthenticationController(DatabaseContext databaseContext, ITokenBuilder tokenBuilder)
        {
            this.databaseContext = databaseContext;
            this.tokenBuilder    = tokenBuilder;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 로그인하기 - Login(user)

        /// <summary>
        /// 로그인하기
        /// </summary>
        /// <param name="user">사용자</param>
        /// <returns>액션 결과 태스크</returns>
        [HttpPost("login")]
        public async Task<IActionResult> Login([FromBody]UserModel user)
        {
            UserModel existingUser = await this.databaseContext.User.SingleOrDefaultAsync(u => u.UserName == user.UserName);
            
            if(existingUser == null)
            {
                return NotFound("USER NOT FOUND");
            }
            
            bool isValid = existingUser.Password == user.Password;
            
            if(!isValid)
            {
                return BadRequest("COULD NOT AUTHENTICATE USER");
            }
            
            string token = this.tokenBuilder.BuildToken(user.UserName);
            
            return Ok(token);
        }

        #endregion
        #region 테스트하기 - Test(test)

        /// <summary>
        /// 테스트하기
        /// </summary>
        /// <param name="test">테스트</param>
        /// <returns>액션 결과</returns>
        [HttpPost("test")]
        [Authorize(AuthenticationSchemes = JwtBearerDefaults.AuthenticationScheme)]
        public async Task<IActionResult> Test([FromBody]TestModel test)
        {
            Claim userNameClaim = User.Claims.SingleOrDefault();
            
            if(userNameClaim == null)
            {
                return Unauthorized();
            }
            
            bool exist = await this.databaseContext.User.AnyAsync(user => user.UserName == userNameClaim.Value);
            
            if(!exist)
            {
                return Unauthorized();
            }
            
            return Ok($"SUCCESS : {DateTime.Now:yyyy-MM-dd HH:mm:ss} {test.Value}");
        }

        #endregion
    }
}